@use "sass:map";
@use "variables" as v;

@mixin breakpoint($size) {
  @media (min-width: #{map.get(v.$breakpoints, $size)}) {
    @content;
  }
}

@mixin text-default {
    font-size: 1rem;
    font-weight: 400;
    line-height: 1.5rem;
}
